/*
 * usage: ./dijkstra-checker-easy < their_output
 */
#include <cassert>
#include <cmath>
#include <cstdlib>
#include <iostream>
#include <queue>
#include <unordered_map>
#include <vector>
using namespace std;

typedef vector< unordered_map<int,int> > graph;

bool is_Dijkstra_infinite(const graph &outgoing_edges) {
    // run Bellman-Ford to check for a reachable negative cycle
    int N = outgoing_edges.size();
    vector<long long> distance( N, pow(10,18) );
    distance[0] = 0;

    bool was_change = false;
    for (int loop=0; loop<N; ++loop) {
        was_change = false;
        for (int where=0; where<N; ++where) {
            for (auto edge : outgoing_edges[where]) {
                if (distance[where] + edge.second < distance[edge.first]) {
                    distance[edge.first] = distance[where] + edge.second;
                    was_change = true;
                }
            }
        }
    }
    return was_change;
}

long long run_Dijkstra(const graph &outgoing_edges, long long limit) {
    int N = outgoing_edges.size();
    vector<long long> distance( N, pow(10,18) );
    distance[0] = 0;

    priority_queue< pair<long long,int>, vector< pair<long long,int> >, greater< pair<long long,int> > > PQ;
    PQ.push( {0,0} );

    long long PROCESSED_VERTICES = 0;

    while (!PQ.empty()) {
        long long how_far = PQ.top().first;
        int where = PQ.top().second;
        PQ.pop();
        if (distance[where] != how_far) continue; // discarding an old record

        ++PROCESSED_VERTICES;
        if (PROCESSED_VERTICES == limit) return PROCESSED_VERTICES;

        for (auto edge : outgoing_edges[where]) {
            if (how_far + edge.second < distance[edge.first]) {
                distance[edge.first] = how_far + edge.second;
                PQ.push( { distance[edge.first], edge.first } );
            }
        }
    }
    return PROCESSED_VERTICES;
}

void fail(const string &message) { cout << message << endl; exit(1); }

graph load_graph() {
    
    int N; cin >> N;
    if (!cin) fail("Reading 'n' failed.");
    if (N<1 || N>60) fail("Value of 'n' outside valid range.");

    int M; cin >> M;
    if (!cin) fail("Reading 'm' failed.");
    if (M<0 || M>N*(N-1)) fail("Value of 'm' outside valid range.");

    graph outgoing_edges(N);

    for (int m=0; m<M; ++m) {
        int v_from, v_to, length;
        
        cin >> v_from;
        if (!cin) fail("Reading an edge failed.");
        if (v_from < 0 || v_from >= N) fail("Vertex number outside valid range.");
        
        cin >> v_to;
        if (!cin) fail("Reading an edge failed.");
        if (v_to < 0 || v_to >= N) fail("Vertex number outside valid range.");
        if (v_from == v_to) fail("Self-loops are not allowed.");
        if (outgoing_edges[v_from].count(v_to) > 0) fail("Duplicate edges are not allowed.");

        cin >> length;
        if (!cin) fail("Reading an edge failed (length outside valid range?).");

        outgoing_edges[v_from][v_to] = length;
    }

    return outgoing_edges;
}

int main() {
    graph G = load_graph();
    if (is_Dijkstra_infinite(G)) fail("For the given graph Dijkstra's algorithm never terminates.");
    long long steps = run_Dijkstra(G,10000);
    if (steps < 10000) fail("PROCESSED_VERTICES too small.");
    cout << "OK" << endl;
    return 0;
}
